"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CallTranscriber = void 0;
const TwilioCall_1 = require("../TwilioCall");
const twilio_1 = require("../twilio");
const Debugger_1 = require("../../Debugger");
const Emitter_1 = require("../../Emitter");
class PromptTranscriptionBuilder {
    constructor() {
        this.transcriptions = [];
    }
    add(event) {
        this.transcriptions.push(event);
    }
    clear() {
        this.transcriptions = [];
    }
    merge() {
        if (this.transcriptions.length === 0) {
            return PromptTranscriptionBuilder.EMPTY_TRANSCRIPTION;
        }
        // If all transcripts partial then return last partial
        const areAllPartial = this.transcriptions.every((t) => t.isFinal === false);
        if (areAllPartial) {
            const lastPartial = this.transcriptions[this.transcriptions.length - 1];
            return lastPartial.transcription;
        }
        // Return finals
        const areAllFinals = this.transcriptions.every((t) => t.isFinal);
        if (areAllFinals) {
            return this.transcriptions.map((t) => t.transcription).join(" ");
        }
        // Return Merged finals and last partial
        const lastTranscription = this.transcriptions[this.transcriptions.length - 1];
        const mergedFinals = this.transcriptions
            .filter((t) => t.isFinal)
            .map((t) => t.transcription)
            .join(" ");
        if (lastTranscription.isFinal) {
            return mergedFinals;
        }
        else {
            return `${mergedFinals} ${lastTranscription.transcription}`;
        }
    }
}
PromptTranscriptionBuilder.EMPTY_TRANSCRIPTION = "";
class CallTranscriber extends Emitter_1.TypedEmitter {
    constructor(call, transcriber, completeTranscriptionTimeoutInMs, promptTranscriptionBuilder = new PromptTranscriptionBuilder()) {
        super();
        this.call = call;
        this.transcriber = transcriber;
        this.completeTranscriptionTimeoutInMs = completeTranscriptionTimeoutInMs;
        this.promptTranscriptionBuilder = promptTranscriptionBuilder;
        this.processMessageRef = this.processMessage.bind(this);
        this.closeRef = this.close.bind(this);
        call
            .getStream()
            .on(TwilioCall_1.WebSocketEvents.Message, this.processMessageRef)
            .on(TwilioCall_1.WebSocketEvents.Close, this.closeRef);
        transcriber.on("transcription", this.collectUntilPause.bind(this));
    }
    processMessage(message) {
        const data = JSON.parse(message);
        switch (data.event) {
            case twilio_1.TwilioConnectionEvents.Media:
                this.transcriber.transcribe(Buffer.from(data.media.payload, "base64"));
                break;
        }
    }
    close() {
        this.call
            .getStream()
            .off(TwilioCall_1.WebSocketEvents.Message, this.processMessageRef)
            .off(TwilioCall_1.WebSocketEvents.Close, this.closeRef);
        this.transcriber.close();
    }
    saveAndEmitPartialTranscript() {
        const partialTranscript = this.promptTranscriptionBuilder.merge();
        CallTranscriber.debug("Partial transcript: %s", partialTranscript);
        const event = {
            transcription: partialTranscript,
            isComplete: false,
        };
        this.emit("transcription", event);
    }
    emitFinalTranscript() {
        const finalTranscript = this.promptTranscriptionBuilder.merge();
        CallTranscriber.debug("Final transcript: %s", finalTranscript);
        const event = {
            transcription: finalTranscript,
            isComplete: true,
        };
        this.emit("transcription", event);
    }
    clearTimer() {
        if (this.timeout) {
            clearTimeout(this.timeout);
            this.timeout = undefined;
        }
    }
    collectUntilPause(event) {
        this.promptTranscriptionBuilder.add(event);
        this.saveAndEmitPartialTranscript();
        this.clearTimer();
        this.timeout = setTimeout(() => {
            this.emitFinalTranscript();
            this.promptTranscriptionBuilder.clear();
            this.transcriber.transcriptionComplete();
            this.clearTimer();
        }, this.completeTranscriptionTimeoutInMs);
    }
}
exports.CallTranscriber = CallTranscriber;
CallTranscriber.debug = Debugger_1.Debugger.getPackageDebugger();
