"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.testRunner = void 0;
const TwilioCallServer_1 = require("./testing/TwilioCallServer");
const PluginManager_1 = require("./plugins/PluginManager");
const populateDefaults_1 = require("./configuration/populateDefaults");
const TwilioCaller_1 = require("./call/TwilioCaller");
const IteratingTestAssigner_1 = require("./testing/IteratingTestAssigner");
const MediaStreamRecorder_1 = require("./call/recording/MediaStreamRecorder");
const DefaultTestExecutor_1 = require("./testing/DefaultTestExecutor");
const AudioPlaybackCaller_1 = require("./call/AudioPlaybackCaller");
const consoleUserInterface_1 = require("./testing/reporting/consoleUserInterface");
const CloseServerWhenTestsComplete_1 = require("./testing/CloseServerWhenTestsComplete");
const callConnectedTimeout_1 = require("./testing/callConnectedTimeout");
/**
 * @param config - Configuration used for setting up the tests
 */
const testRunner = (config) => async (call, ivrTest) => {
    config = populateDefaults_1.populateDefaults(config);
    const tests = Array.isArray(ivrTest) ? ivrTest : [ivrTest];
    const userInterface = consoleUserInterface_1.consoleUserInterface();
    const pluginManager = new PluginManager_1.PluginManager([
        new CloseServerWhenTestsComplete_1.CloseServerWhenTestsComplete(),
        userInterface,
        callConnectedTimeout_1.callConnectedTimeout(config, userInterface),
        MediaStreamRecorder_1.mediaStreamRecorderPlugin(config),
    ]);
    pluginManager.initialise();
    const testExecutor = new DefaultTestExecutor_1.DefaultTestExecutor(config.transcriber, config.completeTranscriptionTimeoutInMs);
    const callServer = new TwilioCallServer_1.TwilioCallServer(config.dtmfGenerator, new IteratingTestAssigner_1.IteratingTestAssigner(tests), testExecutor);
    const serverUrl = await callServer.listen(config.localServerPort);
    pluginManager.serverListening(callServer);
    const caller = Buffer.isBuffer(call)
        ? new AudioPlaybackCaller_1.AudioPlaybackCaller()
        : new TwilioCaller_1.TwilioCaller(config.twilioClient);
    const calls = Promise.all(tests.map(() => caller
        .call(call, config.publicServerUrl || serverUrl)
        .then((callRequested) => pluginManager.callRequested(callRequested, tests.length))
        .catch((error) => {
        pluginManager.callRequestErrored(new Error(error));
        throw error;
    })));
    return new Promise((resolve, reject) => {
        calls
            .then(() => {
            callServer.on("stopped", resolve);
            callServer.on("error", reject);
        })
            .catch((error) => {
            callServer
                .stop()
                .catch((err) => err && console.error(err))
                .finally(() => reject(error));
        });
    });
};
exports.testRunner = testRunner;
