"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const testRunner_1 = require("./testRunner");
const get_port_1 = __importDefault(require("get-port"));
const events_1 = require("events");
const ws_1 = __importDefault(require("ws"));
const wait_for_expect_1 = __importDefault(require("wait-for-expect"));
const inOrder_1 = require("./testing/test/inOrder");
const waitForConnection = async (ws) => new Promise((resolve) => ws.on("open", resolve));
const TwilioPacketGenerator = {
    sendMedia: (ws, data) => {
        const payload = {
            event: "media",
            media: {
                payload: data.toString("base64"),
            },
        };
        ws.send(JSON.stringify(payload));
    },
};
class TranscriberTestDouble extends events_1.EventEmitter {
    close() {
        //Intentionally empty
    }
    transcribe() {
        //Intentionally empty
    }
    produceTranscriptionEvent(transcription) {
        const event = { transcription, isFinal: true };
        this.emit("transcription", event);
    }
    transcriptionComplete() {
        //Intentionally empty
    }
}
describe("Test Runner", () => {
    let callServerPort;
    let twilioClient;
    let commonConfig;
    let ws;
    beforeEach(async () => {
        twilioClient = {
            calls: {
                create: jest.fn(),
            },
        };
        callServerPort = await get_port_1.default();
        commonConfig = {
            localServerPort: callServerPort,
            twilioClient: twilioClient,
            dtmfGenerator: { generate: jest.fn() },
            transcriber: () => new TranscriberTestDouble(),
        };
    });
    afterEach(() => {
        if (ws && ![ws.CLOSED, ws.CLOSING].includes(ws.readyState)) {
            ws.close();
        }
    });
    test("HTTPS public server URL converted to WSS URL in TWIML", async () => {
        twilioClient.calls.create.mockRejectedValue(new Error());
        const runner = await testRunner_1.testRunner({
            ...commonConfig,
            publicServerUrl: "https://example.test/",
        });
        try {
            await runner({ from: "", to: "" }, { name: "", test: inOrder_1.inOrder([]) });
        }
        catch (err) {
            /* Intentionally ignore*/
        }
        expect(twilioClient.calls.create).toBeCalledWith(expect.objectContaining({
            twiml: '<?xml version="1.0" encoding="UTF-8"?><Response><Connect><Stream url="wss://example.test/"><Parameter name="from" value=""/><Parameter name="to" value=""/></Stream></Connect></Response>',
        }));
    });
    test("HTTP public server URL converted to WS URL in TWIML", async () => {
        twilioClient.calls.create.mockRejectedValue(new Error());
        const runner = await testRunner_1.testRunner({
            ...commonConfig,
            publicServerUrl: "http://example.test/",
        });
        try {
            await runner({ from: "", to: "" }, { name: "", test: inOrder_1.inOrder([]) });
        }
        catch (err) {
            /* Intentionally ignore*/
        }
        expect(twilioClient.calls.create).toBeCalledWith(expect.objectContaining({
            twiml: '<?xml version="1.0" encoding="UTF-8"?><Response><Connect><Stream url="ws://example.test/"><Parameter name="from" value=""/><Parameter name="to" value=""/></Stream></Connect></Response>',
        }));
    });
    test("twilio called with phone-numbers and TWIML", async () => {
        twilioClient.calls.create.mockRejectedValue(new Error());
        const call = {
            from: "test-from-number",
            to: "test-to-number",
        };
        try {
            await testRunner_1.testRunner(commonConfig)(call, { name: "", test: inOrder_1.inOrder([]) });
        }
        catch (err) {
            /* Intentionally ignore*/
        }
        expect(twilioClient.calls.create).toBeCalledWith({
            from: "test-from-number",
            to: "test-to-number",
            twiml: `<?xml version="1.0" encoding="UTF-8"?><Response><Connect><Stream url="ws://[::]:${callServerPort}/"><Parameter name="from" value="test-from-number"/><Parameter name="to" value="test-to-number"/></Stream></Connect></Response>`,
        });
    });
    test("server closed when failure making call", async () => {
        twilioClient.calls.create.mockRejectedValue(new Error("Error Occurred"));
        await expect(() => testRunner_1.testRunner(commonConfig)({ from: "", to: "" }, { name: "", test: inOrder_1.inOrder([]) })).rejects.toThrowError(new Error("Error Occurred"));
    });
    test("Call Server closed when test finishes", async () => {
        twilioClient.calls.create.mockResolvedValue(undefined);
        const transcriber = new TranscriberTestDouble();
        jest.spyOn(transcriber, "transcribe").mockImplementation(() => {
            transcriber.produceTranscriptionEvent("hello world");
        });
        const config = {
            ...commonConfig,
            completeTranscriptionTimeoutInMs: 1,
            transcriber: () => transcriber,
        };
        const runner = testRunner_1.testRunner(config);
        const runnerPromise = runner({ from: "", to: "" }, { name: "", test: inOrder_1.inOrder([]) });
        // Wait for calls to be made
        await wait_for_expect_1.default(() => {
            expect(twilioClient.calls.create).toBeCalled();
        });
        // Simulate Twilio connecting a call's stream
        ws = new ws_1.default(`ws://[::]:${callServerPort}/`);
        await waitForConnection(ws);
        TwilioPacketGenerator.sendMedia(ws, Buffer.from([0, 1, 2, 3]));
        await runnerPromise;
        await wait_for_expect_1.default(() => expect(ws.readyState).toBe(ws.CLOSED));
    });
    // test("individual call times out if call not connected after predefined time", () => {
    // No audio is received from the call
    //   * No media events received
    //   * No transcription events (what if music is playing)
    // No media is sent to the call
    // Timeout on a per-call basis
    // expect(true).toBe(false);
    // });
    // test("individual test times out if no transcription after predefined time", () => {
    // No audio is received from the call
    //   * No media events received
    //   * No transcription events (what if music is playing)
    // No media is sent to the call
    // Timeout on a per-call basis
    // expect(true).toBe(false);
    // });
});
