"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const TestInstanceClass_1 = require("./TestInstanceClass");
const events_1 = require("events");
const jest_when_1 = require("jest-when");
const inOrder_1 = require("./inOrder");
const TwilioCall_1 = require("../../call/TwilioCall");
const when_1 = require("./conditions/when");
const then_1 = require("./conditions/then");
class WsTestDouble extends events_1.EventEmitter {
    constructor(sendMock = jest.fn()) {
        super();
        this.sendMock = sendMock;
        this.readyState = 1;
    }
    // eslint-disable-next-line no-dupe-class-members
    send(data, cbOrOptions, cb) {
        this.sendMock(data, cbOrOptions, cb);
    }
}
describe("Then response", () => {
    test("payload from Then directive sent to Twilio when condition is met", async () => {
        const transcriptionHandler = new events_1.EventEmitter();
        const testWithSingleCondition = {
            name: "",
            test: inOrder_1.inOrder([
                {
                    whenPrompt: when_1.contains("Hello"),
                    then: then_1.press("123"),
                },
            ]),
        };
        const dtmfBuffer = Buffer.from([0, 1, 2, 3]);
        const dtmfGenerator = {
            generate: jest.fn(),
        };
        jest_when_1.when(dtmfGenerator.generate).calledWith("123").mockReturnValue(dtmfBuffer);
        const callWebSocket = new WsTestDouble();
        const call = new TwilioCall_1.TwilioCall(callWebSocket, dtmfGenerator);
        new TestInstanceClass_1.TestInstanceClass(call, transcriptionHandler, testWithSingleCondition);
        callWebSocket.emit("message", JSON.stringify({ event: "start", streamSid: "test-stream-sid" }));
        await new Promise((resolve) => setTimeout(resolve, 200));
        transcriptionHandler.emit("transcription", {
            transcription: "Hello World",
            isComplete: true,
        });
        expect(callWebSocket.sendMock).toHaveBeenCalledWith(JSON.stringify({
            event: "media",
            streamSid: "test-stream-sid",
            media: {
                payload: dtmfBuffer.toString("base64"),
            },
        }), undefined, undefined);
    });
});
describe("When conditions", () => {
    let call;
    let transcriptionHandler;
    beforeEach(() => {
        transcriptionHandler = new events_1.EventEmitter();
        call = new TwilioCall_1.TwilioCall(new WsTestDouble(), {
            generate: jest.fn(),
        });
    });
    test("passed event emitted if no conditions", () => {
        const testWithEmptyConditions = {
            name: "",
            test: inOrder_1.inOrder([]),
        };
        const handler = new TestInstanceClass_1.TestInstanceClass(call, transcriptionHandler, testWithEmptyConditions);
        const testPassedListener = jest.fn();
        handler.on("testPassed", testPassedListener);
        transcriptionHandler.emit("transcription", {
            transcription: "Hello World",
            isComplete: true,
        });
        expect(testPassedListener).toHaveBeenCalled();
    });
    test("passed event emitted if all conditions pass", () => {
        const testWithTwoCondition = {
            name: "",
            test: inOrder_1.inOrder([
                {
                    whenPrompt: when_1.contains("1"),
                    then: then_1.doNothing(),
                },
                {
                    whenPrompt: when_1.contains("2"),
                    then: then_1.doNothing(),
                },
            ]),
        };
        const handler = new TestInstanceClass_1.TestInstanceClass(call, transcriptionHandler, testWithTwoCondition);
        const testPassedListener = jest.fn();
        handler.on("testPassed", testPassedListener);
        transcriptionHandler.emit("transcription", {
            transcription: "Test transcript 1",
            isComplete: true,
        });
        transcriptionHandler.emit("transcription", {
            transcription: "Test transcript 2",
            isComplete: true,
        });
        expect(testPassedListener).toHaveBeenCalled();
    });
    test("failure event emitted if all but last condition fails", () => {
        const testWithTwoConditions = {
            name: "",
            test: inOrder_1.inOrder([
                {
                    whenPrompt: when_1.contains("1"),
                    then: then_1.doNothing(),
                },
                {
                    whenPrompt: when_1.contains("3"),
                    then: then_1.doNothing(),
                },
            ]),
        };
        const handler = new TestInstanceClass_1.TestInstanceClass(call, transcriptionHandler, testWithTwoConditions);
        const testFailedListener = jest.fn();
        handler.on("testFailed", testFailedListener);
        transcriptionHandler.emit("transcription", {
            transcription: "Test transcript 1",
            isComplete: true,
        });
        transcriptionHandler.emit("transcription", {
            transcription: "Test transcript 2",
            isComplete: true,
        });
        expect(testFailedListener).toHaveBeenCalled();
    });
    test("failure event emitted if first condition fails", () => {
        const twoConditions = {
            name: "",
            test: inOrder_1.inOrder([
                {
                    whenPrompt: when_1.contains("3"),
                    then: then_1.doNothing(),
                },
                {
                    whenPrompt: when_1.contains("2"),
                    then: then_1.doNothing(),
                },
            ]),
        };
        const handler = new TestInstanceClass_1.TestInstanceClass(call, transcriptionHandler, twoConditions);
        const testFailedListener = jest.fn();
        handler.on("testFailed", testFailedListener);
        transcriptionHandler.emit("transcription", {
            transcription: "Test transcript 1",
            isComplete: true,
        });
        transcriptionHandler.emit("transcription", {
            transcription: "Test transcript 2",
            isComplete: true,
        });
        expect(testFailedListener).toHaveBeenCalled();
    });
});
